"""
Budgie Pi Configuration App
Copyright © 2021-2022 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <https://www.gnu.org/licenses/>.
"""


# General
AUTOSTART = "Enable or disable autostart of the configuration tool"

# Layouts Tab
LAYOUTS_TAB = "Configure different display options for smaller screens"
STANDARD_RES = ("Recommended for screens with\n"
                "vertical resolutions of 900 and above")
COMPACT_RES = ("Recommended for small screens with\n"
               "vertical resolutions of at least 768")
MINI_RES = ("Recommended for small screens with\n"
            "vertical resolutions of less than 768")
APPLY_LAYOUT = ("Apply the new layout.\n"
                "Logout/Login required to complete")

# Overclock Tab
OVERCLOCK_TAB = "Configure clock speed and view CPU temperature"
CPU_TEMP = ("Current CPU temperature. The Pi will throttle at temperatures "
            "around 80°C. Overclocking may affect the throttle temp.")
CPU_SPEEDS = ("Higher speeds will increase performance, but may increase "
              "heat or cause instability. Pi400 cannot be set below 1.8GHz")
SPEED_BUTTON = ("Apply the new clock speed.\n"
                "The new speed will take effect after reboot")
NO_CONFIGTXT = ("File '/boot/firmware/config.txt' not found.\n"
                "Overclock disabled - Is this a valid Pi?")

# Remote Tab
REMOTE_TAB = "Options to configure remote access to this device"
REFRESH_IP = ("Updates the IP if you have changed connection\n"
              "method, or connected to a new network.")
FINDMYPI_SERVER = ("Enable FindMyPi UDP server on app startup. This is not "
                   "needed when using nmap on the client")
AUTOLOGIN = ("Enable automatic login, and disable the lock screen to avoid "
             "issues with remote access. See Release Notes for more info.")
XRDP_BUTTON = ("Enable / disable Budgie Pi's XRDP access. This should not "
               "be used when Gnome's RDP is enabled.")
XRDP_NOT_INSTALLED = ("Budgie Pi provides an alternative xrdp option.\n"
                      "This will install the xrdp package.")
SSH_BUTTON = ("Enable or disable SSH access (Remote Login) through Budgie "
              "Control Center Sharing options.")
SSH_NOT_INSTALLED = ("SSH access is not currently available. This will "
                     "install OpenSSH to enable this option in Budgie Desktop "
                     "Settings.")
VNC_BUTTON = ("Enable VNC access to this device, VNC allows screen sharing. "
              "Enabling / Disabling VNC may take a minute to complete.")
VNC_NOT_INSTALLED = ("Budgie Pi provides an alternative vnc option.\n"
                     "This will install the x11vnc package.")

# Display Tab
DISPLAY_TAB = "Configure Video Modes and Video Memory"
VIDEO_MODE = ("KMS mode is recommended for attached displays. FKMS is "
              "recommended for remote access. Legacy is non-accelerated.")
GPU_MEMORY = ("Select the amout of memory to reserve for video usage. Too "
              "large a value may hurt performance if it is not needed.")
UPDATE_VIDEO = ("Apply the selected video mode\nReboot required for changes "
                "to take effect")
UPDATE_MEMORY = ("Apply the selected memory setting\n"
                 "Reboot required for changes to take effect")
NO_PIBOOTCTL = ("Unable to make changes to the Display options.\n"
                "This is not a Pi, or pibootctl may not be installed.")

# Findmypi
NMAP_BUTTON = ("When enabled, nmap is used to scan the network for Pis.\n"
               "If disabled, it scans for Pis running the FindMyPi server.")
REFRESH_BUTTON = "Rescan the network for Raspberry Pis"
COPYIP_BUTTON = "Copy the currently selected IP address to the clipboard"
SSH_FINDPI = "Open a SSH connection to the selected machine"
SSH_ENTRY = "Enter the username you wish to use for the SSH session"


def add(widget, label, status):
    widget.connect("enter-notify-event", set_label, label, status)
    widget.connect("leave-notify-event", clear_label, label)


def set_label(widget, event, label, status):
    label.set_text(status)


def clear_label(widget, event, label):
    label.set_text("")



import os
import sys
from lib import hint
import argparse
from modules.remote import Remote
from modules.overclock import Overclock
from modules.layout import Layout
from modules.display import Display
from findmypi.findmypi import FindMyPi
import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio


"""
Budgie Pi Configuration App
Copyright © 2021-2022 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <https://www.gnu.org/licenses/>.
"""


class Handler:
    def on_ConfigWindow_destroy(self, *args):
        gsettings.set_boolean('runarmconfig',
                              startlogincheckbutton.get_active())
        Gtk.main_quit()

    def on_ApplyButton_clicked(self):
        if miniradiobutton.get_active():
            layoutstyle.apply(Layout.MINI)
        elif compactradiobutton.get_active():
            layoutstyle.apply(Layout.COMPACT)
        else:
            layoutstyle.apply(Layout.STANDARD)

    def on_ConfigNotebook_switch_page(self, page, num):
        gsettings.set_int('lastpage', num)

    def on_RefreshIP_clicked(self):
        remote.refresh_ip()


def check_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--force-findpi-mode", action="store_true",
                        help="Start up in FindMyPi mode")
    parser.add_argument("--force-arm-mode", action="store_true",
                        help="Start in Config App mode")
    parser.add_argument("--get-model-info", action="store_true",
                        help="Display the model information")
    parser.add_argument("--force-model", choices=['CM4', '4', '400'],
                        help="Recognize this machine as the specifed model")
    parser.add_argument("--model", action='append')
    parser.add_argument("--cpuinfo", action='append',
                        help="When --cpuinfo is used with --model, a matching"
                        + " CPU will be recognized as the specified model")
    parsed_args = parser.parse_args()
    pi_model_list = []
    models = parsed_args.model if parsed_args.model is not None else []
    cpu_infos = parsed_args.cpuinfo if parsed_args.cpuinfo is not None else []
    if len(cpu_infos) != len(models):
        parser.error("number of --cpuinfo and --model arguments must match")
    else:
        for index, model in enumerate(models):
            pi_model_list.append([model, cpu_infos[index]])
    return parsed_args, pi_model_list


args, model_list = check_args()
path = os.path.dirname(os.path.abspath(__file__))
resource = Gio.Resource.load(path + '/org.ubuntubudgie.armconfig.gresource')
Gio.resources_register(resource)
builder = Gtk.Builder.new_from_resource(
    '/org/ubuntubudgie/armconfig/config.ui')
window = builder.get_object("ConfigWindow")
window.show_all()

standardradiobutton = builder.get_object("StandardRadioButton")
compactradiobutton = builder.get_object("CompactRadioButton")
miniradiobutton = builder.get_object("MiniRadioButton")

startlogincheckbutton = builder.get_object("StartLoginCheckButton")
gsettings = Gio.Settings.new('org.ubuntubudgie.armconfig')
startlogincheckbutton.set_active(gsettings.get_boolean('runarmconfig'))
notebook = builder.get_object('ConfigNotebook')
notebook.set_current_page(gsettings.get_int('lastpage'))

layoutstyle = Layout(builder)
overclock = Overclock(builder, args.force_model, model_list)

builder.connect_signals(Handler)
app_statuslabel = builder.get_object("AppStatusLabel")
hint.add(startlogincheckbutton, app_statuslabel, hint.AUTOSTART)

if args.get_model_info:
    pimodel = overclock.get_pimodel_line().split(':')
    if len(pimodel) > 1:
        sys.exit(pimodel[1].strip() + " " + overclock.get_model_memory())
    else:
        sys.exit("Unable to get Pi model info. Is this a Raspberry Pi?")

if ((overclock.pi_model is None and not args.force_arm_mode)
        or args.force_findpi_mode):
    findmypi = FindMyPi(builder)
else:
    remote = Remote(builder)
    display = Display(builder, overclock)

Gtk.main()



#!/usr/bin/python3 -tt
# -*- coding: utf-8 -*-

__license__   = 'GPLv2 or any later version'

from subprocess import *

import argparse 
import ast
import configparser
import logging as log
import os
import sys
import tempfile
import subprocess
from gi.repository import Gio

SYS_KEYS = '/usr/share/budgie-desktop/keycontrol/remove-keys/'
DCONF_KEY_PATH = '/org/gnome/settings-daemon/plugins/media-keys/'

class BudgieKeyError(Exception):
    '''A generic error handler that does nothing.'''
    pass


def parseDconf(conf, is_string=False):
    _cparse = configparser.ConfigParser(interpolation=None)
    if is_string == True:
        _cparse.read_string(conf)
    else:
        _cparse.read(conf)
    return _cparse

def keyNameExists(key, conf):
    if key != 'DEFAULT':
       try:
           exists = conf[key]
           return True
       except KeyError:
           return False

def getKeyFiles(key_dir):
    '''return a list of filenames from a given directory, 
       conforming to our key filenaming standard (*.dconf)'''
    _file_list = []
    for dirpath,_,filenames in os.walk(key_dir):
        for f in filenames:
            if f.endswith('.dconf'):
                _full_path = os.path.abspath(os.path.join(dirpath, f))
                _file_list.append(os.path.abspath(os.path.join(dirpath, f)))
            else:
                log.debug('skipping %s, doesnt end in .dconf' % f)
    return _file_list

if __name__ == '__main__':
    aparse = argparse.ArgumentParser(description = "load custom keybindings via dconf keys")
    aparse.add_argument(
            '-v', '--verbose',  action = 'store_true', 
            help = 'increased output and informational/error messages'
    )
    aparse.add_argument(
            '-D', '--dconf-dir', default=SYS_KEYS,
	    help = 'directory of dconf custom-keybinding files to process'
    )
    args = aparse.parse_args()

    # setup logging
    if args.verbose:
        log.basicConfig(level='DEBUG', format='%(levelname)s: %(message)s')
    else:
        log.basicConfig(level='INFO', format='%(levelname)s: %(message)s')

    
    dconf_files = getKeyFiles(args.dconf_dir)

    if len(dconf_files) == 0:
        log.warning('no files found to process in (' + str(args.dconf_dir) + ', ' + str(args.dconf_file) + ')')
        quit()

    # get our current dconf custom keybinding config for comparison
    curr_conf = parseDconf(check_output(['dconf', 'dump', '/org/gnome/settings-daemon/plugins/media-keys/']).decode('ascii'), True)

    # prepare a single conf object to compile all our new keys into
    out_conf = configparser.ConfigParser(interpolation=None)
    key_ref = []

    # parse all the custom keys in our conf files, and create our dconf entries from them
    settings = Gio.Settings.new('org.gnome.settings-daemon.plugins.media-keys')
    list = settings.get_strv('custom-keybindings')
    
    for dconf_file in dconf_files:
        try:
            dconf_key = parseDconf(dconf_file)
        except configparser.MissingSectionHeaderError:
            log.error('%s is missing a Section Header' % dconf_file)
        except configparser.ParsingError:
            log.error('%s has an incorrectly formatted (or missing) element in section %s' % (dconf_file,key_name))
        
        for key_name in dconf_key:
            if key_name != 'DEFAULT':
                if keyNameExists(key_name, curr_conf) and key_name != '/':
                    key_name = key_name.split('/')[1]
                    log.debug('dconf has key %s' % (key_name))
    
                    settings_args = ['gsettings',
                                     'reset-recursively',
                                     'org.gnome.settings-daemon.plugins.media-keys.custom-keybinding:/org/gnome/settings-daemon/plugins/media-keys/custom-keybindings/'+key_name+'/']
                    subprocess.run(settings_args)

                    list.remove('/org/gnome/settings-daemon/plugins/media-keys/custom-keybindings/'+key_name+'/')


    settings.set_strv('custom-keybindings', list)



#!/usr/bin/env python3
import subprocess
import os

path = "org.gnome.settings-daemon.plugins.media-keys.custom-keybinding:" + \
       "/org/gnome/settings-daemon/plugins/media-keys/custom-keybindings/"

def change_keybinding():
    keyinfo = subprocess.check_output([
        "dconf", "dump",
        "/org/gnome/settings-daemon/plugins/media-keys/custom-keybindings/",
    ]).decode("utf-8").split("\n\n")


    for s in keyinfo:
        if all([check_binding in s, check_command in s]):
            custom = s.splitlines()[0].replace("[", "").replace("]", "")
            command = [
                "gsettings", "set", path + custom + "/", "command", target_command
            ]
            try:
                subprocess.Popen(command)
            except Exception:
                pass

check_binding = "<Super>e"
check_command = "nautilus --new-window %U"
check_name = "Show File Browser"
target_command = 'nemo'
change_keybinding()

check_binding = "<Super>d"
check_command = "/usr/lib/budgie-desktop/plugins/budgie-hcorners/showdesktop"
check_name = "Hide/Show desktop"
target_command = '/usr/share/budgie-desktop/showdesktop/showdesktop'
change_keybinding()



import socket
import threading
import time
import queue
import subprocess
import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gdk, GLib, Pango


"""
Budgie Pi FindMyPi
Author: Samuel Lane
Copyright © 2021-2022 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <https://www.gnu.org/licenses/>.
"""


class FindMyPIClient:
    # Gets a list of Pis by one of two methods: scanning the network and
    # checking mac addresses, or having a UDP server running on the PI.
    # Used by the FindMyPiTreeView, but designed to be used independently

    PI_MACS = [
        ["4B or newer", ['dc:a6:32:']],
        ["3B+ or earlier", ['b8:27:eb:']],
        ["", ['28:cd:c1', 'e4:5f:01:']]
    ]

    def __init__(self, port=32323):
        self.ip_prefix = self._get_ip_prefix()
        self.max_threads = 255  # max number of threads to scan network
        self.timeout = 1        # how many seconds to wait for each ip
        self.port = port        # port to look for the UDP server on
        self.last_scan = 0.0    # time of last nmap scan
        self.prior_search = []  # the previous list of Pis found via scan

    def _get_ip_prefix(self):
        # Gets the client's IP prefix
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        try:
            s.connect(('10.255.255.255', 1))
            IP = s.getsockname()[0].split('.')
        except Exception:
            IP = ['127', '0', '0']
        finally:
            s.close()
        IP_prefix = "{}.{}.{}.".format(IP[0], IP[1], IP[2])
        return IP_prefix

    def _get_model_by_mac(self, mac_addr):
        # Returns Pi model, based on mac address
        for model in self.PI_MACS:
            if any(mac.lower() in mac_addr.lower() for mac in model[1]):
                return " ".join(["Raspberry PI", model[0]])
        return ''

    def _run_nmap(self):
        # Run nmap to try to expose all mac addresses on network
        nmap = [GLib.find_program_in_path('nmap'), '-sn',
                self.ip_prefix+"0/24"]
        try:
            subprocess.check_output(nmap,
                                    stderr=subprocess.STDOUT).decode("utf-8")
        except Exception as e:
            print(e)

    def _run_arp(self):
        # Look at /proc/net/arp for the list of mac addresses for IPs
        output = []
        try:
            with open('/proc/net/arp', 'r') as arp_list:
                line = arp_list.readline().rstrip()
                while line:
                    if "00:00:00" not in line and "IP address" not in line:
                        output.append(line)
                    line = arp_list.readline().rstrip()
        except IOError as e:
            print("Error", e)
        return output

    def _answers_ping(self, ip):
        # Returns true if a Pi responds to ping
        # Used when arp temporarily doesn't show a pi, check before removing
        ping = ['timeout', '2', 'ping', '-c1', ip]
        try:
            output = subprocess.Popen(ping, stdout=subprocess.PIPE)
            output.communicate()
            if output.returncode == 0:
                return True
            else:
                return False
        except OSError:
            return False

    def get_list_from_mac(self):
        # scans using nmap / arp to look at mac address
        pi_list = []

        # after specified interval, run nmap
        now = time.time()
        if (now - self.last_scan) > 600.0:
            self.last_scan = now
            self._run_nmap()
            # slight delay between nmap and arp sometimes increases accuracy
            time.sleep(2)

        output = self._run_arp()
        for line in output:
            results = line.split()
            ip = results[0]
            mac = ''
            for i in range(len(results)):
                # make sure we are looking at a mac address
                if results[i].count(':') == 5:
                    mac = results[i]
            model = self._get_model_by_mac(mac)
            if mac != '' and model != '':
                item = [ip, mac, model]
                pi_list.append(item)

        # if a PI was on the last scan, but not the current scan, try to
        # ping it before removing (in case arp doesn't find it temporarily)
        for pi in self.prior_search:
            if pi not in pi_list and self._answers_ping(pi[0]):
                pi_list.append(pi)

        self.prior_search = pi_list.copy()

        return pi_list

    def _scan(self, ip_queue, results):
        # Scans an IP from the queue and checks if it is a PI
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.settimeout(self.timeout)
        while True:
            ip = ip_queue.get()
            if ip is None:
                break
            try:
                sock.sendto("Are you a PI?".encode("utf-8"), (ip, self.port))
                response = sock.recvfrom(4096)
                msgdata = response[0].decode("utf-8")
                sender = (response[1][0])
                msg = "{},{}".format(sender, msgdata)
                # If local machine is running on a PI with server active,
                # don't add the localhost ip if there is a real IP also.
                if sender == "127.0.0.1":
                    if self.ip_prefix == "127.0.0.":
                        # If you are not connected to a network and you a PI
                        # scanning your own IP (why?), it's ok to add it
                        results.put(msg)
                    else:
                        # If 127.0.0.1 returns a result but have a real network
                        # IP also, do nothing for 127.0.0.1
                        pass
                else:
                    results.put(msg)
            except (socket.timeout, OSError):
                pass
        sock.close()

    def get_list_from_server(self):
        # Starts 'max_threads' number of threads to search for PIs
        self.ip_prefix = self._get_ip_prefix()
        ip_queue = queue.Queue()
        results = queue.Queue()
        pool = []
        threadcount = self.max_threads
        # if IP address is 127.0.0.1, just scan it, don't scan entire subnet
        if self.ip_prefix == '127.0.0.':
            threadcount = 1
            ip_queue.put('127.0.0.1')
        else:
            for i in range(1, 255):
                ip_queue.put(self.ip_prefix + '{0}'.format(i))
        for i in range(threadcount):
            t = threading.Thread(target=self._scan, args=(ip_queue, results),
                                 daemon=True)
            pool.append(t)
        for i in range(threadcount):
            ip_queue.put(None)
        for p in pool:
            p.start()
        for p in pool:
            p.join()
        pi_list = []
        while not results.empty():
            match = results.get().split(',')
            # if a machine answers twice (ie. if it's on wifi and ethernet)
            # only add the ip once (unlikely, but possible)
            if match not in pi_list:
                pi_list.append(match)
        return pi_list


class FindMyPiTreeView (Gtk.TreeView):
    # Gtk.Treeview that contains a list of discovered PIs, and continually
    # updates the list, and allows manual updating of list.

    def __init__(self, port=32323, method='server', frequency=120):
        super().__init__()

        self.port = port
        self.findpi = FindMyPIClient(port=self.port)
        self.pi_liststore = Gtk.ListStore(str, str, str)
        self.set_model(self.pi_liststore)
        self.set_size_request(290, 150)
        self.set_method(method)
        self.fields = ['IP Address', 'Host ID', 'Model']
        self.frequency = frequency
        self.started = False

        for i, field in enumerate(self.fields):
            cell = Gtk.CellRendererText()
            if i == 0:
                cell.props.weight_set = True
                cell.props.weight = Pango.Weight.BOLD
            col = Gtk.TreeViewColumn(field, cell, text=i)
            col.set_sort_column_id(i)
            if i == 2:
                col.set_min_width(300)
            else:
                col.set_min_width(130)
            self.append_column(col)

    def set_method(self, method='server'):
        self.use_arp = True if method == 'mac' else False
        self.method_changed = True

    def start(self):
        # Starts the thread that updates the treeview
        if not self.started:
            self.started = True
            self.pi_liststore.append(['Searching', '', ''])
            self.thread = threading.Thread(target=self._search, daemon=True)
            self.thread.start()

    def _search(self):
        # Thread that automatically refreshes the list
        while True:
            self.refresh_list()
            time.sleep(self.frequency)

    def _update_list(self, pi_list):
        to_remove = []
        # If we switch methods we need to clear the list or else they will
        # display with the old information from the previous scans
        if self.method_changed:
            self.method_changed = False
            self.pi_liststore.clear()

        # Search liststore, generate list of iterations to be removed
        # Also removes existing PIs from the list of PIs to add
        iter_child = self.pi_liststore.get_iter_first()
        while iter_child:
            in_list = False
            ip = self.pi_liststore.get_value(iter_child, 0)
            for pi in pi_list:
                if pi[0] == ip:
                    in_list = True
                    pi_list.remove(pi)
            if not in_list:
                to_remove.append(iter_child)
            iter_child = self.pi_liststore.iter_next(iter_child)
        for child in to_remove:
            self.pi_liststore.remove(child)
        # Adds missing PIs to liststore
        for pi in pi_list:
            self.pi_liststore.append(pi)

    def get_list_from_server(self):
        # Returns list of connected PIs using FindMyPIClient
        return self.findpi.get_list_from_server()

    def get_list_from_mac(self):
        # Returns list of connected PIs using mac address search
        return self.findpi.get_list_from_mac()

    def refresh_list(self):
        # Refresh the treeview.  Can be called manually
        if self.use_arp:
            pi_list = self.findpi.get_list_from_mac()
        else:
            pi_list = self.findpi.get_list_from_server()
        GLib.idle_add(self._update_list, pi_list)

    def get_value_at_col(self, column):
        # Returns the value of the specified column currently selected row
        value = ""
        view_selection = self.get_selection()
        (model, tree_iter) = view_selection.get_selected()
        if tree_iter is not None:
            value = model.get_value(tree_iter, column)
        return value



#!/usr/bin/env python3

import socket
from select import select


"""
Budgie Pi FindMyPi
Author: Samuel Lane
Copyright © 2021-2022 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <https://www.gnu.org/licenses/>.
"""


class FindMyPIServer:
    def __init__(self, port=32323):
        self.port = port
        self.host = socket.gethostname()
        self.model = self._get_model()
        self.ip = self.get_ip()
        self.run = False

    def update_ip(self):
        self.ip = self.get_ip()

    def _get_model(self):
        model = "unknown"
        try:
            with open("/proc/cpuinfo") as cpufile:
                cpuinfo = cpufile.readlines()
                for line in cpuinfo:
                    info = line.split(':')
                    if info[0].strip() == "Model":
                        model = info[1].strip()
                    elif info[0].strip() == "model name":
                        model = info[1].strip()
        except Exception as e:
            print('Unable to get CPU info:', e)
        return model

    def get_ip(self):
        # Return current IP
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        try:
            s.connect(('10.255.255.255', 1))
            IP = s.getsockname()[0]
        except Exception:
            IP = '127.0.0.1'
        finally:
            s.close()
        return IP

    def run_server(self):
        self.run = True
        self.ip = self.get_ip()
        self.hostname = socket.gethostname()
        self.udp = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        try:
            self.udp.bind(('0.0.0.0', self.port))
        except OSError:
            print("Socket could not be opened")
        else:
            print("FindMyPI server started")
            self.input = [self.udp]
            while self.run:
                inputready, outputready, exceptready = select(self.input,
                                                              [], [], 2)
                for s in inputready:
                    if s == self.udp:
                        data, addr = s.recvfrom(4096)
                        msg = data.decode("utf-8")
                        if msg == "Are you a PI?":
                            reply = "{},{}".format(self.host, self.model)
                            sent = s.sendto(reply.encode(), addr)
                    else:
                        print("unknown socket:")
            self.udp.close()


if __name__ == "__main__":
    findmypiserver = FindMyPIServer(32323)
    findmypiserver.run_server()
